SUBROUTINE ForceToForce(FFORCE,NATOM,NTYPEATOM,NSUB,antalmod,ITTER,MASS,SSZ,OLDFREQ,QAMP,WVECTOR,WVECTORS,SYMBLOCK,SYMOPF,NSYMREL,QPCOM,DEG,FOUM,NDISP,DISPL,BLANDAD)
        ! This subroutine calculates:
        ! F(k),q(k,s),W(k,s) --> V(R) (force constants).
        ! symmetrization of V(R)
        ! V(R) + displacements ---> F(R) --> "FORCES" file.
        
        ! Some explanation of variables
        ! NSUB = SSZ(1)*SSZ(2)*SSZ(3), Number of atoms in each sublattice
        ! NATOM = Number of atoms in cell
        ! NTYPEATOM = Number of different ions in cell
        ! antalmod = 3*NATOM 

        IMPLICIT NONE
        EXTERNAL :: cmatmul,invert
        COMPLEX*16, EXTERNAL :: cdotp
        INTEGER, INTENT(IN) :: NATOM,NTYPEATOM,SSZ(3),antalmod,ITTER,NSYMREL,NSUB,NDISP,FOUM(NSUB,NSUB),DEG(NSUB)
        COMPLEX*16,INTENT(IN) ::FFORCE(NSUB,antalmod),WVECTOR(NSUB,antalmod,antalmod),WVECTORS(NSUB,antalmod,antalmod),QAMP(NSUB,antalmod)
        DOUBLE PRECISION, INTENT(IN) ::MASS(10),DISPL(24)
        DOUBLE PRECISION, INTENT(INOUT) :: OLDFREQ(NSUB,antalmod)
        DOUBLE PRECISION, INTENT(IN) :: SYMOPF(NATOM,48,3,3),QPCOM(NSUB,3)
        INTEGER, INTENT(IN) :: SYMBLOCK(NATOM,NATOM,NSYMREL,4)
        LOGICAL,INTENT(INOUT) :: BLANDAD(NSUB,antalmod)
        ! Some local variables are defined here:

        DOUBLE PRECISION :: VSUB(NSUB,NATOM,NATOM,3,3) !forceconstants in different sublattices
        DOUBLE PRECISION :: NVSUB(NSUB,NATOM,NATOM,3,3) !forceconstants in different sublattices
        DOUBLE PRECISION ::MATR(3,3),MATR1(3,3),MATR2(3,3),MATR3(3,3),R(3),K(3),FORCE(NDISP,NSUB,antalmod)
        DOUBLE PRECISION :: RECGITTER(3,3),MASSUB(NATOM),VTOT(NSUB,antalmod,antalmod),U(antalmod),DELTAV(antalmod,antalmod)
        COMPLEX*16 :: W(antalmod),W1(antalmod),W2(antalmod),LAMDA(NSUB,antalmod,antalmod)
        COMPLEX*16 :: WWAVY(NSUB,antalmod,antalmod),DYN(NSUB,antalmod,antalmod),F(antalmod)
        COMPLEX*16 :: CMAT(antalmod,antalmod),CMAT1(antalmod,antalmod),CMAT2(antalmod,antalmod),CMAT3(antalmod,antalmod)
        COMPLEX*16 :: DCMAT(antalmod,antalmod),VBIG(NSUB,antalmod,antalmod)
        INTEGER :: i,j,l,NMASS(NTYPEATOM),NATOMTOT,mm,nn,DEGENERANCY,jm,n1,n2,n3,n4,n5,n6
        DOUBLE PRECISION :: basis(NATOM,3),LATT,VECT(3)
        DOUBLE PRECISION, ALLOCATABLE :: RS(:,:)
        DOUBLE PRECISION, PARAMETER :: pi =3.1415926535897932384626433832795028841970d0
        CHARACTER(LEN=3) :: DUMMY
        COMPLEX*16, PARAMETER :: IM = (0.0d0, 1.0d0 )


        !Here the atom positions in the POSCAR file are read
        
        NATOMTOT = SSZ(1)*SSZ(2)*SSZ(3)*NATOM
        ALLOCATE(RS(NATOMTOT,3))
        OPEN(10,FILE='POSCAR',STATUS='OLD',ACTION='READ')
        READ(10,*) DUMMY
        READ(10,*) LATT
        READ(10,*) MATR(1,1),MATR(1,2),MATR(1,3)
        READ(10,*) MATR(2,1),MATR(2,2),MATR(2,3)
        READ(10,*) MATR(3,1),MATR(3,2),MATR(3,3)
        READ(10,*) NMASS
        READ(10,*) DUMMY
        DO j=1,NATOMTOT
                READ(10,*) RS(j,1),RS(j,2),RS(j,3)
        ENDDO
        close(10)

        ! Here the masses of the different atom types are put into
        ! the array MASSUB
        jm = 0
        DO i=1,NTYPEATOM
                j= NMASS(i)/NSUB
                DO l=1,j
                        MASSUB(l+jm) = MASS(i)
                ENDDO
                jm = j+jm
        ENDDO

        ! Here the lattice vectors of the supercell are put into columns

        MATR = TRANSPOSE(MATR)

        ! Here the reciprocal lattice vectors are calculated:

        MATR2(:,1) = MATR(:,1)/(1.0d0*SSZ(1))
        MATR2(:,2) = MATR(:,2)/(1.0d0*SSZ(2))
        MATR2(:,3) = MATR(:,3)/(1.0d0*SSZ(3))
        
        call invert(MATR2,RECGITTER)
        
        RECGITTER = TRANSPOSE(RECGITTER)

        ! Here the atomic positions (Direct coordinates) are 
        ! stored

        DO i=1,NATOM
                n1 = 1+(i-1)*NSUB
                basis(i,:) = RS(n1,:)
        ENDDO
        
        ! Here the phase-factors exp(R*K) are multiplied with the
        ! eigenvectors W in order to make the calculation of the
        ! force-constants more straight forward.

        DO i=1,NSUB
                K = QPCOM(i,:)
                K = MATMUL(RECGITTER,K)
                DO j=1,antalmod
                       DO l=1,NATOM
                            R = MATMUL(MATR,basis(l,:))
                            n1 = 1+3*(l-1)
                            n2 = 2+3*(l-1)
                            n3 = 3+3*(l-1)
                            WWAVY(i,n1,j)=WVECTOR(i,n1,j)*exp(2.0d0*pi*IM*DOT_PRODUCT(R,K))
                            WWAVY(i,n2,j)=WVECTOR(i,n2,j)*exp(2.0d0*pi*IM*DOT_PRODUCT(R,K))
                            WWAVY(i,n3,j)=WVECTOR(i,n3,j)*exp(2.0d0*pi*IM*DOT_PRODUCT(R,K))
                       ENDDO
                ENDDO
         ENDDO
                           

         ! Here the diagonal eigenvalue matrices ( containing the phonon
         ! frequencies**2 on the diagonal ) are calculated for the
         ! different comensurate k-vectors.
         99 FORMAT(3(F15.10),X,I3)
  kloop: DO i=1,NSUB
                !print*,'K=',i
                LAMDA(i,:,:) = (0.0d0,0.0d0)
      modeloop: DO j=1,antalmod
                        DEGENERANCY = DEG(i)
               degloop: DO l=1,DEG(i)
                                F = FFORCE(FOUM(i,l),:)
                                W = WWAVY(FOUM(i,l),:,j)
                                W = CONJG(W)
                                IF ( QAMP(FOUM(i,l),j) .NE. (0.0d0,0.0d0) ) THEN
                                        LAMDA(i,j,j) = LAMDA(i,j,j)-(1.0d0/sqrt(1.0d0*NSUB))*(cdotp(antalmod,W,F)*CONJG(QAMP(FOUM(i,l),j)) &
                                        & /(ABS(QAMP(FOUM(i,l),j)))**2 )
                                ELSE
                                        IF ( i .NE. 1 ) THEN
                                                DEGENERANCY = DEGENERANCY -1
                                        ENDIF
                                                
                                ENDIF
                         ENDDO degloop
                         !IF ( DEGENERANCY .GT. 0 )WRITE(*,FMT=99)LAMDA(i,j,j)/(DEGENERANCY*1.0d0),OLDFREQ(i,j),DEGENERANCY
                         !IF ( DEGENERANCY .LE. 0 )print*,DEGENERANCY
                         IF ( DEGENERANCY .GT. 0 ) LAMDA(i,j,j) = 0.50d0*(LAMDA(i,j,j)/(DEGENERANCY*1.0d0)+&
                                                   & CONJG(LAMDA(i,j,j)/(DEGENERANCY*1.0d0)))
                         IF ( DEGENERANCY .GT. 0 ) BLANDAD(i,j) = .TRUE.
                         IF ( DEGENERANCY .LE. 0 ) LAMDA(i,j,j) = OLDFREQ(i,j)
                         
                ENDDO modeloop
           ENDDO kloop
           
           ! Here the mean-value of OMEG**2 is calculated as the
           ! arethmetic mean-value over all ITERATIONS > 2
           
           IF ( ITTER .GE. 2) THEN
                DO i=1,NSUB
                        DO j=1,antalmod
                                LAMDA(i,j,j)=(LAMDA(i,j,j)+OLDFREQ(i,j)*(ITTER-1))/(1.0d0*ITTER)
                                OLDFREQ(i,j) = DBLE(LAMDA(i,j,j))
                        ENDDO
                 ENDDO
            ENDIF
           
           ! Here the dynamical matrices for all commensurate k-vectors are 
           ! calculated

           DO i=1,NSUB
               CMAT1 = LAMDA(i,:,:)
               CMAT2 = WVECTORS(i,:,:)
               CMAT3 = CONJG(CMAT2)
               CMAT3 = TRANSPOSE(CMAT3)
               call cmatmul(antalmod,CMAT1,CMAT3,CMAT)
               call cmatmul(antalmod,CMAT2,CMAT,DCMAT)
               ! Imposing Hermicity:
               DYN(i,:,:) = 0.5d0*(DCMAT+CONJG(TRANSPOSE(DCMAT)))
           ENDDO
           
           ! Here we apply the phase-factors to the different sublocks
           ! of the dynamical matrices so that when we fourier transform
           ! back to the force-constant matrices we only have to sum
           ! the 3NATOMx3NATOM force-constant matrices over the first
           ! sublattice having b = (0,0,0)
           IF ( NATOM .GT. 1 ) THEN
            DO l=1,NSUB
                K = QPCOM(l,:)
                K = MATMUL(RECGITTER,K)
                DO i=1,NATOM
                   DO j=1,NATOM
                      IF ( i .NE. j ) THEN
                        R = basis(i,:)-basis(j,:)
                        R = MATMUL(MATR,R)
                        n1 = 1+3*(i-1)
                        n2 = 1+3*(j-1)
                        n3 = 1+3*(i-1)
                        n4 = 2+3*(j-1)
                        n5 = 1+3*(i-1)
                        n6 = 3+3*(j-1)
                        DYN(l,n1,n2)=DYN(l,n1,n2)*exp(2.0d0*pi*IM*DOT_PRODUCT(R,K))
                        DYN(l,n3,n4)=DYN(l,n3,n4)*exp(2.0d0*pi*IM*DOT_PRODUCT(R,K))
                        DYN(l,n5,n6)=DYN(l,n5,n6)*exp(2.0d0*pi*IM*DOT_PRODUCT(R,K)) 
                        n1 = 2+3*(i-1)
                        n2 = 1+3*(j-1)
                        n3 = 2+3*(i-1)
                        n4 = 2+3*(j-1)
                        n5 = 2+3*(i-1)
                        n6 = 3+3*(j-1)
                        DYN(l,n1,n2)=DYN(l,n1,n2)*exp(2.0d0*pi*IM*DOT_PRODUCT(R,K))
                        DYN(l,n3,n4)=DYN(l,n3,n4)*exp(2.0d0*pi*IM*DOT_PRODUCT(R,K))
                        DYN(l,n5,n6)=DYN(l,n5,n6)*exp(2.0d0*pi*IM*DOT_PRODUCT(R,K))
                        n1 = 3+3*(i-1)
                        n2 = 1+3*(j-1)
                        n3 = 3+3*(i-1)
                        n4 = 2+3*(j-1)
                        n5 = 3+3*(i-1)
                        n6 = 3+3*(j-1)
                        DYN(l,n1,n2)=DYN(l,n1,n2)*exp(2.0d0*pi*IM*DOT_PRODUCT(R,K))
                        DYN(l,n3,n4)=DYN(l,n3,n4)*exp(2.0d0*pi*IM*DOT_PRODUCT(R,K))
                        DYN(l,n5,n6)=DYN(l,n5,n6)*exp(2.0d0*pi*IM*DOT_PRODUCT(R,K))
                      ENDIF
                   ENDDO
                ENDDO
             ENDDO
           ELSE
            DYN(1,:,:)= CMPLX(0.0d0,0.0d0)
           ENDIF
           ! Here the force-constant matrices are calculated for the
           ! vectors in the supercell belonging to sublattice 1 
           ! i.e atom_basisvector =(0,0,0) 

           DO i = 1,NSUB
                VBIG(i,:,:) = CMPLX(0.0d0, 0.0d0)
                R = RS(i,:)
                R = MATMUL(MATR,R)
                DO j=1,NSUB
                   K = QPCOM(j,:)
                   K = MATMUL(RECGITTER,K)
                        VBIG(i,:,:) = VBIG(i,:,:)+(1.0d0/(1.0d0*NSUB))*DYN(j,:,:)*exp(2.0d0*pi*IM*DOT_PRODUCT(R,K))
                        !VBIG(i,:,:) = VBIG(i,:,:)+0.50d0*(1.0d0/(1.0d0*NSUB))*&
                        !&((DYN(j,:,:)+CONJG(DYN(j,:,:)))*cos(2.0d0*pi*DOT_PRODUCT(R,K))+&
                        !&IM*(DYN(j,:,:)-CONJG(DYN(j,:,:)))*sin(2.0d0*pi*DOT_PRODUCT(R,K)))
                ENDDO
           ENDDO
           
           !Here the force-constant matrix is separated in to 
           !the matrices VSUB( sublattice i, sublattice j )
           ! so that each sub-block can be symetrizised separetly
           55 FORMAT(3(F30.10,2X))
           66 FORMAT(6(F15.10,X))
           DO l=1,NSUB
                !print*,'R=',l

                DO i=1,NATOM
                 DO j=1,NATOM
                   
                   n1 = 1+3*(i-1)
                   n2 = 1+3*(j-1)
                   n3 = 1+3*(i-1)
                   n4 = 2+3*(j-1)
                   n5 = 1+3*(i-1)
                   n6 = 3+3*(j-1)
                   VSUB(l,i,j,1,1) = DBLE(VBIG(l,n1,n2))
                   VSUB(l,i,j,1,2) = DBLE(VBIG(l,n3,n4)) 
                   VSUB(l,i,j,1,3) = DBLE(VBIG(l,n5,n6))

                   n1 = 2+3*(i-1)
                   n2 = 1+3*(j-1)
                   n3 = 2+3*(i-1)
                   n4 = 2+3*(j-1)
                   n5 = 2+3*(i-1)
                   n6 = 3+3*(j-1)

                   VSUB(l,i,j,2,1) = DBLE(VBIG(l,n1,n2)) 
                   VSUB(l,i,j,2,2) = DBLE(VBIG(l,n3,n4)) 
                   VSUB(l,i,j,2,3) = DBLE(VBIG(l,n5,n6))

                   n1 = 3+3*(i-1) 
                   n2 = 1+3*(j-1)
                   n3 = 3+3*(i-1)
                   n4 = 2+3*(j-1)
                   n5 = 3+3*(i-1)
                   n6 = 3+3*(j-1)

                   VSUB(l,i,j,3,1) = DBLE(VBIG(l,n1,n2)) 
                   VSUB(l,i,j,3,2) = DBLE(VBIG(l,n3,n4)) 
                   VSUB(l,i,j,3,3) = DBLE(VBIG(l,n5,n6))
                   VSUB(l,i,j,:,:) = sqrt(MASSUB(i)*MASSUB(j))*VSUB(l,i,j,:,:)
                   !print*,'(i,j)=',i,j
                   !WRITE(*,FMT=55)VSUB(l,i,j,1,:)
                   !WRITE(*,FMT=55)VSUB(l,i,j,2,:)
                   !WRITE(*,FMT=55)VSUB(l,i,j,3,:)
                   !print*,'---------------------'
                 ENDDO
               ENDDO
            ENDDO
            
            ! Symmetrization of the force-constant matrix

            DO i=1,NATOM
                DO j=1,NATOM
                     DO l=1,NSUB
                        NVSUB(l,i,j,:,:) = 0.0d0
                        DEGENERANCY = 0
                        DO nn=1,NSYMREL
                                IF ( SYMBLOCK(i,j,nn,4) .GT. 0 .AND.SYMBLOCK(i,j,nn,1) .EQ. l ) THEN
                                   MATR1 = SYMOPF(j,SYMBLOCK(i,j,nn,3),:,:)
                                   MATR2 = TRANSPOSE(MATR1)
                                   MATR3 = VSUB(SYMBLOCK(i,j,nn,2),i,j,:,:)
                                   NVSUB(l,i,j,:,:)=NVSUB(l,i,j,:,:)+MATMUL(MATR2,MATMUL(MATR3,MATR1))
                                   DEGENERANCY = DEGENERANCY+1
                                ENDIF
                        ENDDO
                        IF ( DEGENERANCY .GT. 0 ) NVSUB(l,i,j,:,:) = NVSUB(l,i,j,:,:)/(1.0d0*DEGENERANCY)
                        IF ( DEGENERANCY .EQ. 0 ) NVSUB(l,i,j,:,:) = VSUB(l,i,j,:,:)                        
                        !print*,'R=',l
                        !print*,'(i,j)=',i,j
                        !WRITE(*,FMT=55)NVSUB(l,i,j,1,:)
                        !WRITE(*,FMT=55)NVSUB(l,i,j,2,:)
                        !WRITE(*,FMT=55)NVSUB(l,i,j,3,:)
                        !print*,'---------------------'
                     ENDDO
              ENDDO
           ENDDO
           
           ! Here the (3*NATOM)X(3*NATOM) force-constant matrix is assembled
           ! together

           DO l=1,NSUB
                DO i=1,NATOM
                 DO j=1,NATOM

                   n1 = 1+3*(i-1)
                   n2 = 1+3*(j-1)
                   n3 = 1+3*(i-1)
                   n4 = 2+3*(j-1)
                   n5 = 1+3*(i-1)
                   n6 = 3+3*(j-1)
                   
                   VTOT(l,n1,n2) =  NVSUB(l,i,j,1,1)
                   VTOT(l,n3,n4) =  NVSUB(l,i,j,1,2)
                   VTOT(l,n5,n6) =  NVSUB(l,i,j,1,3)

                   n1 = 2+3*(i-1)
                   n2 = 1+3*(j-1)
                   n3 = 2+3*(i-1)
                   n4 = 2+3*(j-1)
                   n5 = 2+3*(i-1)
                   n6 = 3+3*(j-1)

                   VTOT(l,n1,n2) =  NVSUB(l,i,j,2,1)
                   VTOT(l,n3,n4) =  NVSUB(l,i,j,2,2)
                   VTOT(l,n5,n6) =  NVSUB(l,i,j,2,3)

                   n1 = 3+3*(i-1)
                   n2 = 1+3*(j-1)
                   n3 = 3+3*(i-1)
                   n4 = 2+3*(j-1)
                   n5 = 3+3*(i-1)
                   n6 = 3+3*(j-1)

                   VTOT(l,n1,n2) =  NVSUB(l,i,j,3,1)
                   VTOT(l,n3,n4) =  NVSUB(l,i,j,3,2)
                   VTOT(l,n5,n6) =  NVSUB(l,i,j,3,3)
                 ENDDO
               ENDDO
            ENDDO
            
           
            ! Here we enforce the sumrule SUM{R}V(R) = D(k=0)
            DELTAV(:,:) = 0.0d0
            DO i=1,NSUB
                DELTAV = DELTAV+VTOT(i,:,:)
            ENDDO
            DO i=1,NATOM
                DO j=1,NATOM
                  DO l=1,3
                        n1 = l+3*(i-1)
                        n2 = 1+3*(j-1)
                        n3 = l+3*(i-1)
                        n4 = 2+3*(j-1)
                        n5 = l+3*(i-1)
                        n6 = 3+3*(j-1)
                        DYN(1,n1,n2)=DYN(1,n1,n2)*sqrt(MASSUB(i)*MASSUB(j))
                        DYN(1,n3,n4)=DYN(1,n3,n4)*sqrt(MASSUB(i)*MASSUB(j))
                        DYN(1,n5,n6)=DYN(1,n5,n6)*sqrt(MASSUB(i)*MASSUB(j))
                  ENDDO
               ENDDO
            ENDDO  
                  
            DELTAV = (DBLE(DYN(1,:,:))-DELTAV)/(1.0d0*NSUB)
            DO i=1,NSUB
                VTOT(i,:,:) = VTOT(i,:,:)+DELTAV
            ENDDO
            ! Here the FORCES on the atoms in the supercell are calculated 
            
            !DO l =1,NSUB
            !    VTOT(l,:,:) = 0.50d0*(VTOT(l,:,:)+TRANSPOSE(VTOT(l,:,:)))
            !ENDDO
            DO l=1,NDISP
                U(:) = 0.0d0
                n1 = 2+4*(l-1)
                n2 = 3+4*(l-1)
                n3 = 4+4*(l-1)
                VECT(1) = DISPL(n1)*LATT
                VECT(2) = DISPL(n2)*LATT
                VECT(3) = DISPL(n3)*LATT
                VECT = MATMUL(MATR,VECT)
                n4 = 1+3*INT(DISPL(1+(l-1)*4)/NSUB)
                n5 = 2+3*INT(DISPL(1+(l-1)*4)/NSUB)
                n6 = 3+3*INT(DISPL(1+(l-1)*4)/NSUB)
                U(n4) = VECT(1)
                U(n5) = VECT(2)
                U(n6) = VECT(3)
                !print*,'DISPL=1',l
                !IF (NATOM .EQ. 2 ) WRITE(*,FMT=66)U
                DO i=1,NSUB
                        FORCE(l,i,:) = -MATMUL(VTOT(i,:,:),U)
                ENDDO
             ENDDO
             ! Here the forces are saved to the FORCES-file

             OPEN(39,FILE='FORCES',ACTION='WRITE')
             1 FORMAT(I3)
             2 FORMAT(I3,X,3(F15.10,2X))
             3 FORMAT(3(F30.10,2X))
             WRITE(39,FMT=1)NDISP
             DO l=1,NDISP
                n1 = 1+(l-1)*4
                nn = DINT(DISPL(n1))
                n2 = 2+4*(l-1)
                n3 = 3+4*(l-1)
                n4 = 4+4*(l-1)
                WRITE(39,FMT=2)nn,DISPL(n2),DISPL(n3),DISPL(n4)
                DO j=1,NATOM
                        DO i=1,NSUB        
                                n1 = 1+3*(j-1)
                                n2 = 2+3*(j-1)
                                n3 = 3+3*(j-1)
                                WRITE(39,FMT=3)FORCE(l,i,n1),FORCE(l,i,n2),FORCE(l,i,n3)
                        ENDDO
                ENDDO
              ENDDO
              close(39)
END SUBROUTINE ForceToForce
